﻿#include-once
#include <WinAPI.au3>
#include <WinAPIFiles.au3>

#Region ;**** zusaetzliche Strukturen ****
Global Const $tagTIMEOUT = 'struct;' & _  ; wird von _SerialCommSetTimeouts() benutzt
		'long ReadIntervalTimeout;' & _
		'long ReadTotalTimeoutMultiplier;' & _
		'long ReadTotalTimeoutConstant;' & _
		'long WriteTotalTimeoutMultiplier;' & _
		'long WriteTotalTimeoutConstant;' & _
		'endstruct'
Global Const $tagDCB = 'struct;' & _  ; wird von _SerialCommSetState() benutzt
		'long DCBlength;' & _
		'long BaudRate;' & _
		'long fBitFields;' & _
		'short wReserved;' & _
		'short XonLim;' & _
		'short XoffLim;' & _
		'byte Bytesize;' & _
		'byte parity;' & _
		'byte StopBits;' & _
		'byte XonChar;' & _
		'byte XoffChar;' & _
		'Byte ErrorChar;' & _
		'Byte EofChar;' & _
		'Byte EvtChar;' & _
		'short wReserved1;' & _
		'endstruct'
#EndRegion ;**** zusaetzliche Strukturen ****

#Region ;**** Funktionen ****
;===============================================================================
; Function Name:   _SerialCommOpenPort($sSerialPort[, $iTimeout][, $iBaudRate][, $iDataBits][, $iParity][, $iStopBits])
; Description:     Oeffnet den angegebenen seriellen Port und setzt die uebergebenen Parameter
; Return Value(s): das Handle zum geoeffneten Port (wird fuer die anderen Funtionen benoetigt und muss am Ende geschlossen werden).
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_OpenPort($sSerialPort, $iTimeout = 1000, $iBaudRate = 9600, $iDataBits = 8, $iParity = 0, $iStopBits = 1)
	If $sSerialPort = '' Then Return SetError(1, 0, 0)
	If StringLeft($sSerialPort, 4) <> '\\.\' Then $sSerialPort = '\\.\' & $sSerialPort
	Local $hPort = _WinAPI_CreateFileEx($sSerialPort, $OPEN_EXISTING, BitOR($GENERIC_READ, $GENERIC_WRITE)) ; seriellen Port oeffnen
	If @error Then Return
	_SerialComm_SetState($hPort, $iBaudRate, $iDataBits, $iParity, $iStopBits) ; Schnittstellen-Parameter uebergeben
	_SerialComm_SetTimeouts($hPort, $iTimeout) ; Timeout uebergeben
	Return $hPort
EndFunc   ;==>_SerialCommOpenPort

;===============================================================================
; Function Name:   _SerialCommClosePort(ByRef $hPort)
; Description:     Schliesst den angegebenen seriellen Port
; Return Value(s): True = Ok, False = Error
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_ClosePort(ByRef $hPort)
	Return _WinAPI_CloseHandle($hPort) ; seriellen Port schliessen
EndFunc   ;==>_SerialCommClosePort

;===============================================================================
; Function Name:   _SerialCommSendData(ByRef $hPort, $tData[, $bWait])
; Description:     sendet die uebergebenen Daten
;                  $tData kann entweder eine Struktur enthalten oder eine Variable
;                  (deren Daten in Binaerdaten umgewandelt und gesendet werden).
; Return Value(s): die Anzahl der gesendeten Bytes
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_SendData(ByRef $hPort, $tData, $bWait = 1)
	If $hPort = 0 Then Return SetError(1, 0, 0)
	If Not IsDllStruct($tData) Then ; wenn $tData keine Struktur ist, dann...
		If $tData = '' Then Return SetError(2, 0, 0) ; bei einem Leerstring die Funktion abbrechen
		Local $dData = $tData
		If Not IsBinary($dData) Then $dData = Binary($dData) ; in Binaerdaten umwandeln
		$tData = DllStructCreate('byte[' & BinaryLen($dData) & ']') ; Byte-Struktur erstellen
		DllStructSetData($tData, 1, $dData) ; die Daten in die Struktur schreiben
	EndIf
	Local $iBytes, $bRet, $iSize = DllStructGetSize($tData) ; Groesse der Struktur ermitteln
	$bRet = _WinAPI_WriteFile($hPort, $tData, $iSize, $iBytes) ; Daten senden
	$tData = 0
	If Not $bRet Then Return SetError(3, 0, 0)
	If $bWait Then _WinAPI_FlushFileBuffers($hPort) ; warten, bis die Daten uebertragen wurden
	Return $iBytes
EndFunc   ;==>_SerialCommSendData

;===============================================================================
; Function Name:   _SerialCommReceiveData(ByRef $hPort, ByRef $tData)
; Description:     empfaengt Daten (in Groesse der uebergebenen Struktur)
; Return Value(s): die Anzahl der empfangenen Bytes und
;                  die Struktur enthaelt die empfangenen Daten (ByRef-Uebergabe)
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_ReceiveData(ByRef $hPort, ByRef $tData)
	Local $iSize = DllStructGetSize($tData), $iBytes, $bRet
	$bRet = _WinAPI_ReadFile($hPort, $tData, $iSize, $iBytes) ; Daten empfangen
	If Not $bRet Then Return SetError(1, 0, 0)
	Return $iBytes
EndFunc   ;==>_SerialCommReceiveData

;===============================================================================
; Function Name:   _SerialCommSetTimeouts(ByRef $hPort[, $iMillis])
; Description:     setzt die Timeouts auf die angegebenen Millisekunden
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_SetTimeouts(ByRef $hPort, $iMillis = 1000)
	If $hPort = 0 Then Return SetError(1, 0, 0)
	Local $tTimeout = DllStructCreate($tagTIMEOUT)
;~ 	DllCall('kernel32.dll', 'long', 'GetCommTimeouts', 'hwnd', $hPort, 'ptr', DllStructGetPtr($tTimeout))
	DllStructSetData($tTimeout, 'ReadTotalTimeoutConstant', $iMillis)
	DllStructSetData($tTimeout, 'WriteTotalTimeoutConstant', $iMillis)
	DllCall('kernel32.dll', 'long', 'SetCommTimeouts', 'hwnd', $hPort, 'ptr', DllStructGetPtr($tTimeout))
	$tTimeout = 0
EndFunc   ;==>_SerialCommSetTimeouts

;===============================================================================
; Function Name:   _SerialCommSetState(ByRef $hPort[, $iBaudRate][, $iDataBits][, $iParity][, $iStopBits])
; Description:     setzt die uebergebenen Parameter
; Author(s):       Oscar (www.autoit.de)
;===============================================================================
Func _SerialComm_SetState(ByRef $hPort, $iBaudRate = 9600, $iDataBits = 8, $iParity = 0, $iStopBits = 1)
	If $hPort = 0 Then Return SetError(1, 0, 0)
	Local $tDcb = DllStructCreate($tagDCB)
;~ 	DllCall('kernel32.dll', 'short', 'GetCommState', 'hwnd', $hPort, 'ptr', DllStructGetPtr($tDcb))
	DllStructSetData($tDcb, 'BaudRate', $iBaudRate)
	DllStructSetData($tDcb, 'Bytesize', $iDataBits)
	DllStructSetData($tDcb, 'Parity', $iParity)
	DllStructSetData($tDcb, 'StopBits', $iStopBits)
	DllCall('kernel32.dll', 'short', 'SetCommState', 'hwnd', $hPort, 'ptr', DllStructGetPtr($tDcb))
	$tDcb = 0
EndFunc   ;==>_SerialCommSetState
#EndRegion ;**** Funktionen ****

